local _,ns = ...

BigBrother = LibStub("AceAddon-3.0"):NewAddon("BigBrother", "AceConsole-3.0", "AceEvent-3.0")
local BigBrother = BigBrother

local L = LibStub("AceLocale-3.0"):GetLocale("BigBrother")

local _G = _G
local select, type, next, ipairs = _G.select, _G.type, _G.next, _G.ipairs

--localize names for lookup
local flaskBuffs = {}
for i,v in ipairs(ns.SpellData.flasks) do
	flaskBuffs[i] = GetSpellInfo(v)
end

local elixirBuffs = {}
for i,v in ipairs(ns.SpellData.elixirsBattle) do
	elixirBuffs[i] = GetSpellInfo(v)
end
for i,v in ipairs(ns.SpellData.elixirsGuardian) do
	elixirBuffs[i] = GetSpellInfo(v)
end

local foodBuffs = {}
for i,v in ipairs(ns.SpellData.foods) do
	foodBuffs[i] = GetSpellInfo(v)
end


function BigBrother:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("BigBrotherDB", {
		profile = {
			autogroups = true,
			groups = { true, true, true, true, true, true, true, true },
			checkFlasks = true,
			checkElixirs = true,
			checkFood = true,
			checkReadyCheck = true,
		}
	})
end

function BigBrother:OnEnable()
	self:RegisterEvent("READY_CHECK")
end

--from BigWigs:GetInstanceDifficulty()
local function GetInstanceSize()
	local _, instanceType, diff, _, _, heroic, dynamic = _G.GetInstanceInfo()
	if instanceType == "raid" and dynamic and heroic == 1 and diff <= 2 then
		diff = diff + 2
	end
	diff = type(diff) == "number" and diff or 1
	
	if diff == 1 or diff == 3 then
		return 10
	elseif diff == 2 or diff == 4 then
		return 25
	else
		return 5
	end
end

local SendChatMessageList
do
	local chatTypes = {
		SELF = true, SAY = true, GROUP = true, PARTY = true, GUILD = true, OFFICIER = true, RAID = true, RAID_WARNING = true, WHISPER = true,
	}
	function SendChatMessageList(title, list, channel)
		channel = channel and string.upper(channel)
		if not channel or not chatTypes[channel] then
			channel = "SELF"
			--BigBrother:Print("Invalid channel, set to \"SELF\".")
		end
		
		local t = {}
		for k in next, list do
			t[#t + 1] = k
		end

		if #t > 0 then
			table.sort(t)
			local names = table.concat(t, ", ")
			if channel == "SELF" then
				BigBrother:Print(("|cffff8040%s (%d)|r: %s"):format(title, #t, names))
			else
				local target = nil
				if channel == "WHISPER" then
					target = _G.UnitName("target") or _G.UnitName("player")
				elseif channel == "GROUP" then
					if _G.GetNumRaidMembers() > 0 then
						channel = "RAID"
					elseif _G.GetNumPartyMembers() > 0 then
						channel = "PARTY"
					else
						return --abort!
					end
				end
				_G.SendChatMessage(("%s (%d): %s"):format(title, #t, names), channel, nil, target)
			end
		end
	end
end

function BigBrother:GetCheckedGroups()
	local groups
	if self.db.profile.autogroups then
		local d = GetInstanceSize()
		if d == 10 then
			groups = { true, true }
		elseif d == 25 then
			groups = { true, true, true, true, true }
		end
	else 
		groups = self.db.profile.groups
	end
	
	return groups
end

do
	local partyUnits = {"player"}
	for i = 1, MAX_PARTY_MEMBERS do
		partyUnits[#partyUnits + 1] = ("party%d"):format(i)
	end
	local raidUnits = {}
	for i = 1, MAX_RAID_MEMBERS do
		raidUnits[#raidUnits + 1] = ("raid%d"):format(i)
	end
	
	function BigBrother:ConsumableCheck(channel)
		local inRaid = _G.GetNumRaidMembers() > 0
		
		local groups = self:GetCheckedGroups()
		
		local missingFlaskList = {}
		local missingElixirList = {}
		local missingFoodList = {}
		
		for i, unit in ipairs(inRaid and raidUnits or partyUnits) do
			local name = _G.UnitName(unit)
			if not name then break end
			local subgroup = inRaid and select(3, _G.GetRaidRosterInfo(i)) or 1
			local online = _G.UnitIsConnected(unit)
			local dead = _G.UnitIsDeadOrGhost(unit)
			
			if online and not dead and groups[subgroup] then
				
				--check flasks/elixirs
				if self.db.profile.checkFlasks then
					local numElixirs = 0
					for _,spellName in next, elixirBuffs do
						if _G.UnitBuff(name, spellName) then
							numElixirs = numElixirs + 1
						end
					end
					if numElixirs == 0 then
						missingFlaskList[name] = true
					elseif numElixirs == 1 then
						missingElixirList[name] = true
					end
					
					if numElixirs == 0 then
						local found = nil
						for _,spellName in next, flaskBuffs do
							if _G.UnitBuff(name, spellName) then
								found = true
								missingFlaskList[name] = nil --since they don't have elixirs
								break
							end
						end
						if not found then
							missingFlaskList[name] = true
						end
					end
				end
				
				--check food
				if self.db.profile.checkFood then
					local found = nil
					for _,spellName in next, foodBuffs do
						if _G.UnitBuff(name, spellName) then
							found = true
							break
						end
					end
					if not found then
						missingFoodList[name] = true
					end
				end
				
			end --if
		end --for
		
		if self.db.profile.checkElixirs then
			SendChatMessageList(L["One Elixir"], missingElixirList, channel)
		end
		if self.db.profile.checkFlasks then
			SendChatMessageList(L["No Flask"], missingFlaskList, channel)
		end
		if self.db.profile.checkFood then	
			SendChatMessageList(L["Not Well Fed"], missingFoodList, channel)
		end
	end
end
	
function BigBrother:READY_CHECK(_, sender)
	if not self.db.profile.checkReadyCheck then return end
	if _G.UnitIsUnit(sender, "player") and _G.GetNumRaidMembers() > 0 then
		self:ConsumableCheck("RAID")
	end
end

-- GLOBALS: table, string